/*
*********************************************
*                                           *
*  NZEV SOUBORU: bmp.cpp                   *
*  AUTOR:         Michal (Sirall) ervenka  *
*  E-MAIL:        cervem5@fel.cvut.cz       *
*                                           *
*********************************************
*/

#include "image.h"

Image *LoadBMP(char *filename) {
	FILE * file;
	BITMAPFILEHEADER fileHeader;
	BITMAPINFOHEADER infoHeader;

	file = fopen(filename, "rb");
	if(!file) {
		return NULL;
	}

	fread(&fileHeader, sizeof(BITMAPFILEHEADER), 1, file);

	if(fileHeader.bfType != bitmapID) {
		fclose(file);
		return NULL;
	}

	fread(&infoHeader, sizeof(BITMAPINFOHEADER), 1, file);

	fclose(file);

	if(infoHeader.biBitCount == 8)
		return Load8BitBMP(filename);
		
	if(infoHeader.biBitCount == 24)
		return Load24BitBMP(filename);

	return NULL;
}

Image *Load24BitBMP(char *filename) {
	FILE * file;
	BITMAPFILEHEADER fileHeader;
	BITMAPINFOHEADER infoHeader;
    Image *image = new Image();


	file=fopen(filename, "rb");
	if(!file) {
		return NULL;
	}

	fread(&fileHeader, sizeof(BITMAPFILEHEADER), 1, file);

	if(fileHeader.bfType != bitmapID) {
		fclose(file);
		return NULL;
	}

	fread(&infoHeader, sizeof(BITMAPINFOHEADER), 1, file);

	if(infoHeader.biBitCount!=24) {
		fclose(file);
		return NULL;
	}

    image->iBpp = 24;
	image->iFormat = GL_RGB;
	image->iWidth = infoHeader.biWidth;
	image->iHeight = infoHeader.biHeight;
 

    // krok
    unsigned int bitsPerLine = image->iWidth * image->iBpp;
    unsigned int bitsToAdd = 0;
    
    if ((bitsPerLine%32) != 0)
        bitsToAdd = 32 - (bitsPerLine%32);

    image->iStride = (bitsPerLine + bitsToAdd) / 8;

	fseek(file, fileHeader.bfOffBits, SEEK_SET);

	image->pData = new unsigned char[image->iStride * image->iHeight];
	if(image->pData == NULL) {
		fclose(file);
		return NULL;
	}

	fread(image->pData, 1, image->iStride * image->iHeight, file);
	fclose(file);

	for(int r = 0; r < image->iHeight; r++) {
		for(int i = 0; i < image->iWidth; i++) {
			image->pData[(r*image->iStride)+i*3] ^= image->pData[(r*image->iStride)+i*3+2] ^= image->pData[(r*image->iStride)+i*3] ^= image->pData[(r*image->iStride)+i*3+2];
		}
	}

	return image;
}

Image *Load8BitBMP(char *filename) {
	FILE * file;
	BITMAPFILEHEADER fileHeader;
	BITMAPINFOHEADER infoHeader;
    Image *image = new Image();


	file=fopen(filename, "rb");
	if(!file) {
		return NULL;
	}

	fread(&fileHeader, sizeof(BITMAPFILEHEADER), 1, file);

	if(fileHeader.bfType != bitmapID) {
		fclose(file);
		return NULL;
	}

	fread(&infoHeader, sizeof(BITMAPINFOHEADER), 1, file);

	if(infoHeader.biBitCount!=8) {
		fclose(file);
		return NULL;
	}

    image->iBpp = 8;
	image->iFormat = GL_COLOR_INDEX;
	image->iWidth = infoHeader.biWidth;
	image->iHeight = infoHeader.biHeight;
    
    // Nacte paletu
    image->bPaletted = true;
    image->iPaletteBpp = 24;
    image->iPaletteFormat = GL_RGB;

    // krok
    unsigned int bitsPerLine = image->iWidth * image->iBpp;
    unsigned int bitsToAdd = 0;
    
    if ((bitsPerLine%32) != 0)
        bitsToAdd = 32 - (bitsPerLine%32);

	image->iStride = (bitsPerLine + bitsToAdd) / 8;

    // Zpracovani palety
    unsigned char *tempPalette = NULL;
    tempPalette = new unsigned char[256*4];
    if (tempPalette == NULL) {
		fclose(file);
		return NULL;
    }
    fread(tempPalette, 256*4, 1, file);
    image->pPalette = new unsigned char[256*3];
    if (image->pPalette == NULL) {
		fclose(file);
		return NULL;
    }
    
    for (int i = 0; i < 256; i++) {
        image->pPalette[i*3+0] = tempPalette[i*4+2];
        image->pPalette[i*3+1] = tempPalette[i*4+1];
        image->pPalette[i*3+2] = tempPalette[i*4+0];
    }    

    delete [] tempPalette;
    tempPalette = NULL;
    

	fseek(file, fileHeader.bfOffBits, SEEK_SET);

	image->pData = new unsigned char[image->iStride * image->iHeight];
	if(image->pData == NULL) {
		fclose(file);
		return NULL;
	}

	fread(image->pData, 1, image->iStride * image->iHeight, file);
	fclose(file);

    image->expandPalette();

	return image;
}
