/*
*********************************************
*                                           *
*  NZEV SOUBORU: image.cpp                 *
*  AUTOR:         Michal (Sirall) ervenka  *
*  E-MAIL:        cervem5@fel.cvut.cz       *
*                                           *
*********************************************
*/

#include "image.h"

Image::Image() {
    this->pData = NULL;
    this->pPalette = NULL;
    this->bPaletted = false;
}

Image::~Image() {
    if (this->pData != NULL)
        delete [] this->pData;
    
    if (this->pPalette != NULL)
        delete [] this->pPalette;
}

void Image::expandPalette() {
    if (this->pPalette == NULL || !this->bPaletted)
        return;
        
    unsigned int bitsPerLine = this->iWidth * this->iPaletteBpp;
    unsigned int bitsToAdd = 0;
    
    if ((bitsPerLine%32) != 0)
        bitsToAdd = 32 - (bitsPerLine%32);

	unsigned int stride = (bitsPerLine + bitsToAdd) / 8;
    unsigned char *data = NULL;
    data = new unsigned char[stride*this->iHeight];
    
    if (data == NULL)
        return;
        
    for (int r = 0; r < this->iHeight; r++) {
        for (int i = 0; i < this->iWidth; i++) {
            unsigned int oldPixel = (r*this->iStride) + i;
            unsigned int dpb = this->iPaletteBpp / 8;           
            unsigned int newPixel = (r*stride) + i*(dpb);
            unsigned int paletteEntry = this->pData[(r*this->iStride) + i];

            for (int j = 0; j < dpb; j++) {
                data[newPixel + j] = this->pPalette[(paletteEntry*dpb) + j];
            }
        }
    }
    
    this->bPaletted = false;
    this->iBpp = this->iPaletteBpp;
    this->iFormat = this->iPaletteFormat;
    this->iStride = stride;
    
    if (this->pData != NULL)
        delete [] this->pData;
        
    this->pData = data;
}
