/*
*********************************************
*                                           *
*  NZEV SOUBORU: tga.cpp                   *
*  AUTOR:         Michal (Sirall) ervenka  *
*  E-MAIL:        cervem5@fel.cvut.cz       *
*                                           *
*********************************************
*/

#include "image.h"

Image *LoadTGA(char *filename) {
	unsigned char uncompTGAHeader[11]		= {0, 2,  0, 0, 0, 0, 0,  0, 0, 0, 0}; // Nekomprimovane tga
	unsigned char compTGAHeader[11]			= {0, 10, 0, 0, 0, 0, 0,  0, 0, 0, 0}; // Koprimovane tga v RLE
	unsigned char uncomp8Bit24TGAHeader[11]	= {1, 1,  0, 0, 0, 1, 24, 0, 0, 0, 0}; // Nekomprimovane tga s barevnou paletou 24 bitu
	unsigned char uncomp8Bit32TGAHeader[11]	= {1, 1,  0, 0, 0, 1, 32, 0, 0, 0, 0}; // Nekomprimovane tga s barevnou paletou 32 bitu

	unsigned char TGAcompare[11];

	FILE * file = NULL;
	file = fopen(filename, "rb");

	if(file == NULL) {
		return FALSE;
	}

    unsigned char picId;
  	fread(&picId, sizeof(unsigned char), 1, file);

  	fread(TGAcompare, sizeof(TGAcompare), 1, file);

 	fclose(file);

	if(memcmp(uncompTGAHeader, TGAcompare, sizeof(uncompTGAHeader)) == 0) {
		return LoadUncompressedTrueColorTGA(filename, picId);
	}

	if(memcmp(compTGAHeader, TGAcompare, sizeof(compTGAHeader)) == 0) {
        return LoadCompressedTrueColorTGA(filename, picId);
	}

	if(	memcmp(uncomp8Bit24TGAHeader, TGAcompare, sizeof(uncomp8Bit24TGAHeader)) == 0 || 
        memcmp(uncomp8Bit32TGAHeader, TGAcompare, sizeof(uncomp8Bit32TGAHeader)) == 0) {
		return LoadUncompressed8BitTGA(filename, picId);
	}

	return NULL;
}

Image *LoadUncompressedTrueColorTGA(char * filename, int idLength) {
	unsigned char uncompTGAHeader[11]   = {0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0};
	unsigned char TGAcompare[11];
	unsigned char header[6];
    bool isUsePictureID = false;

    if (idLength > 0)
        isUsePictureID = true;

	FILE * file = NULL;
    file = fopen(filename, "rb");

	if(file == NULL) {
		return NULL;
	}

    unsigned char picId;
  	fread(&picId, sizeof(unsigned char), 1, file);

	fread(TGAcompare, 1, sizeof(TGAcompare), file);
	if(memcmp(uncompTGAHeader, TGAcompare, sizeof(uncompTGAHeader)) != 0) {
		return NULL;
	}

	fread(header, 1, sizeof(header), file);

    Image *image = new Image();
	image->iWidth =	header[1]*256+header[0];
	image->iHeight = header[3]*256+header[2];
    image->iBpp = header[4];

    unsigned char *pictureIdent = new unsigned char[idLength];
    if (isUsePictureID) {
    	fread(pictureIdent, 1, idLength*sizeof(unsigned char), file);
    }
    delete [] pictureIdent;

	if(image->iWidth <=0 || image->iHeight<=0 || (image->iBpp != 24 && image->iBpp != 32)) {
		fclose(file);
		return NULL;
	}

	if(image->iBpp == 24) {
		image->iFormat = GL_RGB;
	}
	else {
		image->iFormat = GL_RGBA;
	}

    unsigned int bitsPerLine = image->iWidth * image->iBpp;
    unsigned int bitsToAdd = 0;
    
    if ((bitsPerLine%32) != 0)
        bitsToAdd = 32 - (bitsPerLine%32);

    image->iStride = (bitsPerLine + bitsToAdd) / 8;

	image->pData = new unsigned char[image->iStride*image->iHeight];
	if(image->pData == NULL) {
		fclose(file);
		return NULL;
	}

	for(unsigned int i = 0; i < image->iHeight; i++) {
		fread(&image->pData[i*image->iStride], 1, image->iWidth*(image->iBpp/8), file);
	}
	
	fclose(file);

	for(unsigned int i = 0; i < image->iHeight; i++) {
		for(unsigned int j = 0; j < image->iWidth; j++) {
			image->pData[(i*image->iStride) + j*(image->iBpp/8)] ^= image->pData[(i*image->iStride)+j*(image->iBpp/8)+2] ^= 
            image->pData[(i*image->iStride) + j*(image->iBpp/8)] ^= image->pData[(i*image->iStride)+j*(image->iBpp/8)+2];
		}
	}

	return image;
}

Image *LoadUncompressed8BitTGA(char * filename, int idLength) {
	unsigned char uncomp8Bit24TGAHeader[11]   = {1, 1, 0, 0, 0, 1, 24, 0, 0, 0, 0};
	unsigned char uncomp8Bit32TGAHeader[11]   = {1, 1, 0, 0, 0, 1, 32, 0, 0, 0, 0};
	unsigned char TGAcompare[11];
	unsigned char header[6];
    bool isUsePictureID = false;

    if (idLength > 0)
        isUsePictureID = true;

	FILE *file = NULL;
    file = fopen(filename, "rb");

	if(file == NULL) {
		return NULL;
	}

    unsigned char picId;
  	fread(&picId, sizeof(unsigned char), 1, file);

	fread(TGAcompare, 1, sizeof(TGAcompare), file);
	if(	memcmp(uncomp8Bit24TGAHeader, TGAcompare, sizeof(uncomp8Bit24TGAHeader)) != 0 &&
		memcmp(uncomp8Bit32TGAHeader, TGAcompare, sizeof(uncomp8Bit32TGAHeader)) != 0) {
		return NULL;
	}

	fread(header, 1, sizeof(header), file);

    Image *image = new Image();
	image->iWidth =	header[1]*256 + header[0];
	image->iHeight = header[3]*256+header[2];
	image->iBpp = header[4];

    unsigned char *pictureIdent = new unsigned char[idLength];
    if (isUsePictureID) {
    	fread(pictureIdent, 1, idLength*sizeof(unsigned char), file);
    }
    delete [] pictureIdent;

	if(image->iWidth <= 0 || image->iHeight <= 0 || image->iBpp != 8) {
		fclose(file);
		return NULL;
	}
		
    image->bPaletted = true;
    image->iPaletteBpp = TGAcompare[7];
	
	if(image->iPaletteBpp == 24)
		image->iPaletteFormat = GL_RGB;
	if(image->iPaletteBpp == 32)
		image->iPaletteFormat = GL_RGBA;

    image->pPalette = new unsigned char[256*image->iPaletteBpp/8];
	if(image->pPalette == NULL) {
		fclose(file);
		return NULL;
	}

	fread(image->pPalette, 256*image->iPaletteBpp/8, 1, file);

	for(int i = 0; i < 256; i++) {
		image->pPalette[i*(image->iPaletteBpp/8)] ^= image->pPalette[i*(image->iPaletteBpp/8)+2] ^=
        image->pPalette[i*(image->iPaletteBpp/8)] ^= image->pPalette[i*(image->iPaletteBpp/8)+2];
	}
	
    image->iFormat = GL_COLOR_INDEX;

    unsigned int bitsPerLine = image->iWidth * image->iBpp;
    unsigned int bitsToAdd = 0;
    
    if ((bitsPerLine%32) != 0)
        bitsToAdd = 32 - (bitsPerLine%32);

    image->iStride = (bitsPerLine + bitsToAdd) / 8;

	image->pData = new unsigned char[image->iStride * image->iHeight];
	if(image->pData == NULL) {
		fclose(file);
		return NULL;
	}

	for(unsigned int i = 0; i < image->iHeight; i++) {
		fread(&image->pData[i*image->iStride], 1, image->iWidth, file);
	}

	fclose(file);

	return image;
}

Image *LoadCompressedTrueColorTGA(char *filename, int idLength) {
	unsigned char compTGAHeader[11]   = {0, 10, 0, 0, 0, 0, 0, 0, 0, 0, 0};
	unsigned char TGAcompare[11];
	unsigned char header[6];
    bool isUsePictureID = false;

    if (idLength > 0)
        isUsePictureID = true;

	FILE *file = NULL;
 	file = fopen(filename, "rb");

	if(file == NULL) {
		return NULL;
	}

    unsigned char picId;
  	fread(&picId, sizeof(unsigned char), 1, file);

	fread(TGAcompare, 1, sizeof(TGAcompare), file);
	if(memcmp(compTGAHeader, TGAcompare, sizeof(compTGAHeader)) != 0) {
		return NULL;
	}

	fread(header, 1, sizeof(header), file);

    Image *image = new Image();
	image->iWidth = header[1]*256 + header[0];
	image->iHeight = header[3]*256 + header[2];
    image->iBpp = header[4];

    unsigned char *pictureIdent = new unsigned char[idLength];
    if (isUsePictureID) {
    	fread(pictureIdent, 1, idLength*sizeof(unsigned char), file);
    }
    delete [] pictureIdent;

	if(image->iWidth <=0 || image->iHeight <= 0 || (image->iBpp != 24 && image->iBpp != 32)) {
		fclose(file);
		return NULL;
	}

	if(image->iBpp == 24)
        image->iFormat = GL_RGB;
	if(image->iBpp == 32)
		image->iFormat = GL_RGBA;

	unsigned char *tempData = NULL;
    tempData = new unsigned char[image->iWidth*image->iHeight*image->iBpp/8];

	if(tempData == NULL) {
		fclose(file);
		return NULL;
	}

	int pixelCount= image->iWidth*image->iHeight;
	int currentPixel = 0;
	int currentByte = 0;
 	unsigned char *colorBuffer = new unsigned char[image->iBpp/8];

	do {
		unsigned char chunkHeader = 0;

		if(fread(&chunkHeader, sizeof(unsigned char), 1, file) == 0) {
			fclose(file);
			return NULL;
		}

		if(chunkHeader < 128) {
			chunkHeader++;

			for(int counter = 0; counter < chunkHeader; counter++) {
				if(fread(colorBuffer, 1, image->iBpp/8, file) != image->iBpp/8) {
					fclose(file);
					return NULL;
				}

				tempData[currentByte+0] = colorBuffer[2];
				tempData[currentByte+1] = colorBuffer[1];
				tempData[currentByte+2] = colorBuffer[0];

				if(image->iBpp/8==4)
					tempData[currentByte+3] = colorBuffer[3];

				currentByte += image->iBpp/8;
				currentPixel++;

				if(currentPixel > pixelCount) {
					fclose(file);
					return NULL;
				}
			}
		}
		else {
			chunkHeader -= 127;

			if(fread(colorBuffer, 1, image->iBpp/8, file) != image->iBpp/8) {
				fclose(file);
				return NULL;
			}

			for(int counter = 0; counter < chunkHeader; counter++) {
				tempData[currentByte+0] = colorBuffer[2];
				tempData[currentByte+1] = colorBuffer[1];
				tempData[currentByte+2] = colorBuffer[0];

				if(image->iBpp/8 == 4)
					tempData[currentByte+3] = colorBuffer[3];

				currentByte += image->iBpp/8;
				currentPixel++;

				if(currentPixel > pixelCount) {
					fclose(file);
					return NULL;
				}
			}
		}
	} while(currentPixel < pixelCount);

    unsigned int bitsPerLine = image->iWidth * image->iBpp;
    unsigned int bitsToAdd = 0;
    
    if ((bitsPerLine%32) != 0)
        bitsToAdd = 32 - (bitsPerLine%32);

    image->iStride = (bitsPerLine + bitsToAdd) / 8;

    image->pData = new unsigned char[image->iStride*image->iHeight];
	if(image->pData == NULL) {
        fclose(file);
		return false;
	}

	for(unsigned int i = 0; i < image->iHeight; i++) {
		memcpy(&image->pData[(i*image->iStride)], &tempData[i*image->iWidth*image->iBpp/8], image->iWidth*image->iBpp/8);
	}
	
	if(tempData != NULL)
		delete [] tempData;

	if(colorBuffer != NULL)
		delete [] colorBuffer;
    
    fclose(file);

	return image;
}
