#include "grafika.h"
#include "objekty.h"
#include "primitiva/terc.h"
#include "primitiva/zamerovac.h"
#include <cmath>

#define DEG_TO_RAD				(M_PI/180.0)
#define DOHLEDNOST				180.0
#define ZACATEK_MLHY			170.0

const GLdouble UHEL = 0.5;
const GLdouble DELKA_KROKU = 0.4;
const GLdouble SKOK_ZAMEROVACE = 0.1;
const GLfloat VZDALENOST_STRELNIC = 40.0;
const GLfloat POSUN_OBJEKTU = 0.3;
const GLfloat ELEVACE_TERCE = 4.2;
const GLfloat VZDALENOST_TERCE = -13.0;

GLdouble		pozice[3];																			//pozice panacka
GLdouble		smerPohybu[3];																	//smer panacka
GLdouble		uhelPohledu;
GLdouble		uhelOtoceni = 0.0;
GLdouble		poziceKamery;																		//kamera pri pohledu ze strelnice

GLfloat			poziceKralika;
GLfloat			smerKralika;

int					prulezSysla;
GLfloat			smerSysla;
GLfloat			poziceSyslaY;
GLfloat			poziceSyslaX;

GLfloat			velikostSvetla		= 13.0;												//velikost reflektoroveho svetla
GLfloat			poziceReflektor[]	={0.0, 8.0, 7.0, 1.0};				//reflektor
GLfloat			smerReflektoru[]	={0.0, -1.0, -4.5};						//smer svetla reflektoru
GLfloat			poziceSlunce[]		={0.0, 60.0, 0.0, 0.0};				//celkove osvetleni (smerove svetlo)

Puska				zbran;
Zamerovac		muska;
Terc				tercKralik(-VZDALENOST_TERCE*3.0 + 2.0);
Terc				tercSysel(VZDALENOST_TERCE*3.0);
Terc				tercKlasicky(1.0);
Strelnice		strelnice1;
Skybox			prostredi;

float				muskaX, muskaY, muskaZ;

bool				svetloGlobalni = true;
bool				svetloReflektor = false;
GLMmodel		*model_p = NULL;																//ukazatel na vykreslovany model
void (*pohled)(void);																				//ukazatel na funkci, ktera definuje pohled

///inicializacni funkce
void inicializace()
{
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LESS);																			//funkce pro testovani fragmentu
	glShadeModel(GL_SMOOTH);
	glPolygonMode(GL_FRONT, GL_FILL);
	glPolygonMode(GL_BACK, GL_FILL);
	glCullFace(GL_BACK);

	muskaX = 0.0;
	muskaY = 6.0;
	muskaZ = -2.0;

	zbran.zmenOtoceni(180.0);																	//nastaveni spravneho natoceni zbrane
	zbran.zmenPozici(0.0, 4.5, 8.0);													//nastaveni spravne pozice zbrane
	zbran.zmenZvetseni(3.0);
	zbran.zakazVykresleni();

	prostredi.nactiTextury();																	//nacteni textur pro jednotlive prvky
	zbran.nactiTextury();																			//nacteni textury pro vystrel
	tercKralik.nactiTextury("textury/kralik.bmp");
	tercSysel.nactiTextury("textury/sysel.bmp");
	tercKlasicky.nactiTextury("textury/terc.bmp");

	pohled = panorama;

	smerKralika = -POSUN_OBJEKTU;
	poziceKralika = VZDALENOST_STRELNIC - 5.0 + tercKralik.vratDelku();

	smerSysla = POSUN_OBJEKTU;
	prulezSysla = 1;
	poziceSyslaX = -VZDALENOST_STRELNIC + 5.0 - tercSysel.vratDelku();
	poziceSyslaY = ELEVACE_TERCE - 0.1 - tercSysel.vratVysku();

	GLfloat barvaMlhy[]	= {1.0, 1.0, 1.0, 1.0};
	GLfloat ambient[]		= {0.2, 0.2, 0.2, 1.0};
	GLfloat diffuse[]		= {1.0, 1.0, 1.0, 1.0};
	GLfloat specular[]	= {0.5, 0.5, 0.5, 1.0};

	glEnable(GL_LIGHTING);

	glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);								//nastaveni svetla slunce
	glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
	glLightfv(GL_LIGHT0, GL_SPECULAR, specular);

	glLightf(GL_LIGHT1, GL_SPOT_EXPONENT, 1.0);								//nastaveni svetla reflektoru
	glLightfv(GL_LIGHT1, GL_AMBIENT, ambient);
	glLightfv(GL_LIGHT1, GL_DIFFUSE, diffuse);
	glLightfv(GL_LIGHT1, GL_SPECULAR, specular);

	glEnable(GL_FOG);																					//zapnuti a nastaveni mlhy
	glFogi(GL_FOG_MODE, GL_LINEAR);
	glFogfv(GL_FOG_COLOR, barvaMlhy);
	glFogf(GL_FOG_DENSITY, 0.5);
	glHint(GL_FOG_HINT, GL_DONT_CARE);
	glFogf(GL_FOG_START, 5.0);
	glFogf(GL_FOG_END, ZACATEK_MLHY);
}

///otexturuje dany objekt s pouzitim materialu
void setTexture(GLuint ID, Image* image)
{
	glBindTexture(GL_TEXTURE_2D, ID);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexImage2D(GL_TEXTURE_2D, 0, 3, image->iWidth, image->iHeight, 0, image->iFormat, GL_UNSIGNED_BYTE, image->pData);

	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
}

///funkce pro ulozeni matic pred volanim d-listu
void initDraw()
{
	glPushMatrix();
	glPushAttrib(GL_ALL_ATTRIB_BITS);
}

///funkce pro vyvolani d-listu a obnoveni matic
void closeDraw(unsigned dlist)
{
	glCallList(dlist);
	glPopAttrib();
	glPopMatrix();
}

///funkce na vykreslovani okna
void myDisplay()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();

	vykresliScenu();																					//vykresleni sceny

	glPopMatrix();
	glutSwapBuffers();
}

///funkce na zmenu velikosti okna
void myReshape(int sirka, int vyska)
{
	glViewport(0, 0, sirka, vyska);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(90.0, GLdouble(sirka/vyska), 0.1, DOHLEDNOST);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

///funkce vykonavajici se pri necinnosti
void myIdle()
{
	uhelOtoceni += UHEL;
	if(uhelOtoceni > 360.0)
		uhelOtoceni -= 360.0;

	poziceKralika += smerKralika;

	//pokud je kralik uplne vlevo zmen smer posunu doprava
	if (poziceKralika <= VZDALENOST_STRELNIC - 5.0 + tercKralik.vratDelku())
		smerKralika = POSUN_OBJEKTU;

	//pokud je kralik uplne vpravo zmen smer posunu doleva
	if (poziceKralika >= VZDALENOST_STRELNIC + 5.0 - tercKralik.vratDelku())
		smerKralika = -POSUN_OBJEKTU;

	poziceSyslaY += smerSysla;

	//pokud je sysel uplne dole zmen smer posunu nahoru
	if (poziceSyslaY <= ELEVACE_TERCE - 0.2 - tercSysel.vratVysku()) {
		smerSysla = POSUN_OBJEKTU;
		prulezSysla++;
	}

	//pokud je sysel uplne nahore zmen smer posunu dolu
	if (poziceSyslaY >= ELEVACE_TERCE)
		smerSysla = -POSUN_OBJEKTU;

	switch (prulezSysla) {
		case 1:
			poziceSyslaX = -VZDALENOST_STRELNIC + 5.0 - tercSysel.vratDelku();
			break;
		case 2:
			poziceSyslaX = -VZDALENOST_STRELNIC;
			break;
		case 3:
			poziceSyslaX = -VZDALENOST_STRELNIC - 5.0 + tercSysel.vratDelku();
			break;
		default:
			poziceSyslaX = -VZDALENOST_STRELNIC + 5.0 - tercSysel.vratDelku();
			prulezSysla = 1;
	}

	glutPostRedisplay();
}

///funkce obsaravajici menu
void myMenu(int polozka)
{
	switch(polozka) {
		case 1:		//Prochazeni sceny
			pohled = prochazeni;
			glutSpecialFunc(mySpecialKeyboard);
			zbran.zakazVykresleni();
			pozice[0] = 0.0; pozice[1] = 5.0; pozice[2] = 0.0;
			smerPohybu[0] = smerPohybu[1] = 0.0; smerPohybu[2] = -1.0;
			uhelPohledu = -90.0;
			break;
		case 2:		//Panoramaticky pohled
			pohled = panorama;
			uhelOtoceni = 0.0;
			zbran.zakazVykresleni();
			break;
		case 3:		//Palebne stanoviste 1
			pohled = stanoviste;
			glutSpecialFunc(mySpecialKeyboard2);
			zbran.povolVykresleni();
			poziceKamery = VZDALENOST_STRELNIC;
			poziceReflektor[0] = VZDALENOST_STRELNIC;
			zbran.zmenPozici(VZDALENOST_STRELNIC, 4.5, 8.0);
			muskaX = VZDALENOST_STRELNIC;
			muskaY = 5.5;
			break;
		case 4:		//Palebne stanoviste 2
			pohled = stanoviste;
			glutSpecialFunc(mySpecialKeyboard2);
			zbran.povolVykresleni();
			poziceKamery = 0.0;
			poziceReflektor[0] = 0.0;
			zbran.zmenPozici(0.0, 4.5, 8.0);
			muskaX = 0.0;
			muskaY = 5.5;
			break;
		case 5:		//Palebne stanoviste 3
			pohled = stanoviste;
			glutSpecialFunc(mySpecialKeyboard2);
			zbran.povolVykresleni();
			poziceKamery = -VZDALENOST_STRELNIC;
			poziceReflektor[0] = -VZDALENOST_STRELNIC;
			zbran.zmenPozici(-VZDALENOST_STRELNIC, 4.5, 8.0);
			muskaX = -VZDALENOST_STRELNIC;
			muskaY = 5.5;
			break;
		case 6:		//Restart nastaveni
			smerReflektoru[2] = -4.5;
			velikostSvetla = 13.0;
			uhelOtoceni = 0.0;
			svetloGlobalni = true;
			svetloReflektor = false;
			tercKralik.resetZasahu();
			tercSysel.resetZasahu();
			tercKlasicky.resetZasahu();
			zbran.nabij();
			break;
		case 7:		//Konec programu
			exit(0);
			break;
	}
	glutPostRedisplay();
}

///funkce pro ovladani klavesnice
void myKeyboard(unsigned char key, int x, int y)
{
	x = y = 0;
	switch (key) {
		case 'S':																							//zapinani celkoveho osvetleni
		case 's':
			svetloGlobalni = !svetloGlobalni;
			break;
		case 'R':																							//zapinani reflektoru strelnice osvetleni
		case 'r':
			svetloReflektor = !svetloReflektor;
			break;
		case 'A':
		case 'a':
			if (svetloReflektor) smerReflektoru[2] += 0.1;
			break;
		case 'Q':
		case 'q':
			if (svetloReflektor) smerReflektoru[2] -= 0.1;
			break;
		case 'X':
		case 'x':
			if (svetloReflektor) velikostSvetla -= 1.0;
			break;
		case 'C':
		case 'c':
			if (svetloReflektor) velikostSvetla += 1.0;
			break;
		case 'Z':
		case 'z':
			if (zbran.vykreslujeSe())	zbran.zakazVykresleni();
			else											zbran.povolVykresleni();
			break;
		case 49:																							//vystrel ze zbrane - klavesa 1
			if (zbran.vystrel()) {
				tercKralik.trefilSe(poziceKralika, ELEVACE_TERCE, muska.vratX(), muska.vratY());
				tercKlasicky.trefilSe(-5.0 + tercKlasicky.vratDelku(), ELEVACE_TERCE, muska.vratX(), muska.vratY());
				tercKlasicky.trefilSe(0.0, ELEVACE_TERCE, muska.vratX(), muska.vratY());
				tercKlasicky.trefilSe(5.0 - tercKlasicky.vratDelku(), ELEVACE_TERCE, muska.vratX(), muska.vratY());
				tercSysel.trefilSe(poziceSyslaX, poziceSyslaY, muska.vratX(), muska.vratY());
			}
			break;
		case 48:																							//nabiti naboju - klavesa 0
			zbran.nabij();
			break;
		case 27:																							//Esc ukonci aplikaci
			exit(0);
	}
	glutPostRedisplay();
}

///specialni funkce pro klavesnici pri prochazeni scenou
void mySpecialKeyboard(int key, int x, int y)
{
	x = y = 0;
	switch (key) {
		case GLUT_KEY_RIGHT:																	//otoceni smeru pohledu doleva
			uhelPohledu += UHEL*4;
			if(uhelPohledu > 360.0)
				uhelPohledu -= 360.0;
			break;
		case GLUT_KEY_LEFT:																		//otoceni smeru pohledu doprava
			uhelPohledu -= UHEL*4;
			if(uhelPohledu < -360.0)
				uhelPohledu += 360.0;
			break;
		case GLUT_KEY_UP:																			//krok vpred
			pozice[0] += DELKA_KROKU * smerPohybu[0];
			pozice[1] += DELKA_KROKU * smerPohybu[1];
			pozice[2] += DELKA_KROKU * smerPohybu[2];
			break;
		case GLUT_KEY_DOWN:																		//krok vzad
			pozice[0] -= DELKA_KROKU * smerPohybu[0];
			pozice[1] -= DELKA_KROKU * smerPohybu[1];
			pozice[2] -= DELKA_KROKU * smerPohybu[2];
			break;
		case GLUT_KEY_PAGE_UP:																//zvednuti smeru pohledu
			pozice[1] += DELKA_KROKU;
			break;
		case GLUT_KEY_PAGE_DOWN:															//snizeni smeru pohledu
			pozice[1] -= DELKA_KROKU;
			break;
		default: return;
	}
	glutPostRedisplay();
}

///specialni funkce pro klavesnici pri prochazeni scenou
void mySpecialKeyboard2(int key, int x, int y)
{
	x = y = 0;
	switch (key) {
		case GLUT_KEY_RIGHT:																	//pohyb zamerovacem vpravo
			muskaX += SKOK_ZAMEROVACE;
			break;
		case GLUT_KEY_LEFT:																		//pohyb zamerovacem vlevo
			muskaX -= SKOK_ZAMEROVACE;
			break;
		case GLUT_KEY_UP:																			//pohyb zamerovacem nahoru
			muskaY += SKOK_ZAMEROVACE;
			break;
		case GLUT_KEY_DOWN:																		//pohyb zamerovacem dolu
			muskaY -= SKOK_ZAMEROVACE;
			break;
		default: return;
	}
	glutPostRedisplay();
}

///pohled z oci umoznujici strilet na terce
void prochazeni()
{
	smerPohybu[0] = cos(uhelPohledu*DEG_TO_RAD);
	smerPohybu[1] = 0.0;
	smerPohybu[2] = sin(uhelPohledu*DEG_TO_RAD);

	gluLookAt(
		pozice[0], pozice[1]+0.5, pozice[2],															// eye = poloha kamery
		pozice[0]+smerPohybu[0], pozice[1]+0.5, pozice[2]+smerPohybu[2],	// center = bod na ktery koukame (smer pohledu = eye-center)
		0.0, 1.0, 0.0																											// up vektor (orientace kamery podel smeru pohledu)
	);
}

///panoramaticky pohled na scenu
void panorama()
{
	gluLookAt(
		13.0,	15.0,	13.0,		// eye = poloha kamery
		0.0,  0.0,  0.0,		// cenetr = bod na ktery koukame (smer pohledu = eye-center)
		0.0,  1.0,  0.0			// up vektor (orientace kamery podel smeru pohledu)
	);
	glRotatef(uhelOtoceni, 0.0, 1.0, 0.0);
}

///panoramaticky pohled na scenu
void stanoviste()
{
	gluLookAt(
		poziceKamery,	5.5,	8.0,
		poziceKamery,	5.5,	7.5,
		0.0,	1.0,	0.0
	);
}

///vykreslovaci funkce
void vykresliScenu()
{
	pohled();

	if ((pohled != panorama)&&(pohled != prochazeni)) {
		muska.zmenPozici(muskaX, muskaY, muskaZ);
		muska.vykresli();
	}

	if (svetloGlobalni)
		glEnable(GL_LIGHT0);

	if (svetloReflektor)
		glEnable(GL_LIGHT1);

	glLightfv(GL_LIGHT1, GL_POSITION, poziceReflektor);
	glLightfv(GL_LIGHT1, GL_SPOT_DIRECTION, smerReflektoru);
	glLightf(GL_LIGHT1, GL_SPOT_CUTOFF, velikostSvetla);

	glLightfv(GL_LIGHT0, GL_POSITION, poziceSlunce);

	prostredi.vykresli();

	glPushMatrix();
	glTranslated(0.0, 0.2, 0.0);
	strelnice1.vykresli();
	glPopMatrix();

	glPushMatrix();
	glTranslated(-VZDALENOST_STRELNIC, 0.2, 0.0);
	strelnice1.vykresli();
	glPopMatrix();

	glPushMatrix();
	glTranslated(VZDALENOST_STRELNIC, 0.2, 0.0);
	strelnice1.vykresli();
	glPopMatrix();

	tercKralik.zmenPozici(poziceKralika, ELEVACE_TERCE, VZDALENOST_TERCE);
	tercKralik.vykresli();

	tercSysel.zmenPozici(poziceSyslaX, poziceSyslaY, VZDALENOST_TERCE);
	tercSysel.vykresli();

	tercKlasicky.zmenPozici(-5.0 + tercKlasicky.vratDelku(), ELEVACE_TERCE, VZDALENOST_TERCE);
	tercKlasicky.vykresli();

	tercKlasicky.zmenPozici(0.0, ELEVACE_TERCE, VZDALENOST_TERCE);
	tercKlasicky.vykresli();

	tercKlasicky.zmenPozici(5.0 - tercKlasicky.vratDelku(), ELEVACE_TERCE, VZDALENOST_TERCE);
	tercKlasicky.vykresli();

	zbran.vykresli();

	glDisable(GL_LIGHT0);
	glDisable(GL_LIGHT1);
}
