#include "primitiva.h"

///konstruktor ktery nezahrnuje nastaveni materialu
Objekt::Objekt(GLfloat *pole, float koef)
{
	dlist = glGenLists(1);
	koefOdlesku = 0.0;
	koefDeleni = koef;
	memcpy(vrcholy,		pole,		MAX * sizeof(GLfloat));
	memset(ambientni,	0.0,		SLOZKY * sizeof(GLfloat));
	memset(difuzni,		0.0,		SLOZKY * sizeof(GLfloat));
	memset(odlesky,		0.0,		SLOZKY * sizeof(GLfloat));
}

///kopirovaci konstruktor
Objekt::Objekt(Objekt &a)
{
	dlist = glGenLists(1);
	koefOdlesku = a.koefOdlesku;
	koefDeleni = a.koefDeleni;
	memcpy(vrcholy,		a.vrcholy,		MAX * sizeof(GLfloat));
	memcpy(ambientni,	a.ambientni,	SLOZKY * sizeof(GLfloat));
	memcpy(difuzni,		a.difuzni,		SLOZKY * sizeof(GLfloat));
	memcpy(odlesky,		a.odlesky,		SLOZKY * sizeof(GLfloat));
}

///konstruktor tridy zahrnujici nastaveni materialu objektu
Objekt::Objekt(GLfloat *pole, float koef, GLfloat *amb, GLfloat *dif, GLfloat *spec, GLfloat koefSpec)
{
	dlist = glGenLists(1);
	koefOdlesku = koefSpec;
	koefDeleni = koef;
	memcpy(vrcholy,		pole,		MAX * sizeof(GLfloat));
	memcpy(ambientni,	amb,		SLOZKY * sizeof(GLfloat));
	memcpy(difuzni,		dif,		SLOZKY * sizeof(GLfloat));
	memcpy(odlesky,		spec,		SLOZKY * sizeof(GLfloat));
}

///operator prirazeni
Objekt Objekt::operator=(const Objekt &a)
{
	if (this == &a) return *this;

	dlist = a.dlist;
	koefOdlesku = a.koefOdlesku;
	koefDeleni = a.koefDeleni;
	memcpy(vrcholy,		a.vrcholy,		MAX * sizeof(GLfloat));
	memcpy(ambientni,	a.ambientni,	SLOZKY * sizeof(GLfloat));
	memcpy(difuzni,		a.difuzni,		SLOZKY * sizeof(GLfloat));
	memcpy(odlesky,		a.odlesky,		SLOZKY * sizeof(GLfloat));

	return *this;
}

///nastavuje slozky materialu
void Objekt::nastavMaterial(GLfloat *amb, GLfloat *dif, GLfloat *spec, GLfloat koefSpec)
{
	koefOdlesku = koefSpec;
	memcpy(ambientni,	amb,		SLOZKY * sizeof(GLfloat));
	memcpy(difuzni,		dif,		SLOZKY * sizeof(GLfloat));
	memcpy(odlesky,		spec,		SLOZKY * sizeof(GLfloat));
}

///vykresli steny objektu
void Objekt::vykresliSteny()
{
	glNewList(dlist, GL_COMPILE);
		glNormal3f(0.0,-1.0, 0.0);
		rozdelPlochu(vrcholy[0],	vrcholy[1],		vrcholy[2], -vrcholy[0]*2, 0.0, -vrcholy[2]*2);			//spodni
		glNormal3f(0.0, 0.0,-1.0);
		rozdelPlochu(vrcholy[0],	vrcholy[1],		vrcholy[2], -vrcholy[0]*2, vrcholy[13], 0.0);				//predni
		glNormal3f(1.0, 0.0, 0.0);
		rozdelPlochu(vrcholy[3],	vrcholy[4],		vrcholy[5], 0.0, vrcholy[13], -vrcholy[5]*2);				//prava
		glNormal3f(0.0, 0.0,1.0);
		rozdelPlochu(vrcholy[6],	vrcholy[7],		vrcholy[8], -vrcholy[6]*2, vrcholy[13], 0.0);				//zadni
		glNormal3f(-1.0, 0.0, 0.0);
		rozdelPlochu(vrcholy[9],	vrcholy[10],	vrcholy[11], 0.0, vrcholy[13], -vrcholy[11]*2);			//leva
		glNormal3f(0.0, 1.0, 0.0);
		rozdelPlochu(vrcholy[12],	vrcholy[13],	vrcholy[14], -vrcholy[12]*2, 0.0, -vrcholy[11]*2);	//vrchni
	glEndList();
}

///vraci pomer delek stran
float Objekt::pomerStran(const float &strana1, const float &strana2)
{
	if (strana1 == 0 || strana2 == 0)
		return 1.0;

	return int(strana1/strana2);
}

int Objekt::najdiDelitele(int cislo, int delenec)
{
	if (cislo <= 1)
		return 1;

	int i;
	const int ZMENA = 10;
	for (i = 0; i <= ZMENA; i++) {
		if (delenec % (cislo + i) == 0)
			return cislo + i;

		if (delenec % (cislo - i) == 0)
			return cislo - i;
	}

	i = 1;
	while ((delenec % i == 0) && (i < cislo))
		i++;

	return cislo + (i-1);
}

///prepocita pomery stran a zmenseni
void Objekt::spocitejZmenseni(const float &strana1, const float &strana2, float &krok1, float &krok2, float &zmensi1, float &zmensi2)
{
	zmensi2 = najdiDelitele(pomerStran(strana1, strana2), koefDeleni);
	zmensi1 = najdiDelitele(pomerStran(strana2, strana1), koefDeleni);

	krok1 *= zmensi1;
	krok2 *= zmensi2;
}

///rozdeli plochy sten objektu na mensi plosky
void Objekt::rozdelPlochu(float x, float y, float z, const float rozmerX, const float rozmerY, const float rozmerZ)
{
	float krokX	= rozmerX / koefDeleni;
	float krokY	= rozmerY / koefDeleni;
	float krokZ	= rozmerZ / koefDeleni;
	float start = 0.0;
	float zmenseni1 = 1.0;
	float zmenseni2 = 1.0;

	if (rozmerX == 0) {
		spocitejZmenseni(rozmerY, rozmerZ, krokY, krokZ, zmenseni1, zmenseni2);
	}
	else if (rozmerY == 0) {
		spocitejZmenseni(rozmerX, rozmerZ, krokX, krokZ, zmenseni1, zmenseni2);
	}
	else {
		spocitejZmenseni(rozmerX, rozmerY, krokX, krokY, zmenseni1, zmenseni2);
	}

	if (rozmerZ == 0.0)
		start = y;
	else
		start = z;

	zmenseni1 = koefDeleni/zmenseni1;
	zmenseni2 = koefDeleni/zmenseni2;

	for(int i = 0; i < zmenseni1; i++) {
		glBegin(GL_QUAD_STRIP);
			if (rozmerZ == 0.0) {
				y = start;
				glVertex3d(x + krokX, y, z);
				glVertex3d(x, y, z);
			}
			else {
				z = start;
				glVertex3d(x + krokX, y + krokY, z);
				glVertex3d(x, y, z);
			}
			for(int j = 0; j < zmenseni2; j++) {
				if (rozmerZ == 0.0) {
					glVertex3d(x + krokX, y + krokY, z);
					glVertex3d(x, y + krokY, z);
					y += krokY;
				}
				else {
					glVertex3d(x + krokX, y + krokY, z + krokZ);
					glVertex3d(x, y, z + krokZ);
					z += krokZ;
				}
			}
			x += krokX;
			if (rozmerZ != 0.0) y += krokY;
		glEnd();
	}
}

///vykresli dany objekt
void Objekt::vykresli()
{
	glMaterialfv(GL_FRONT, GL_AMBIENT, ambientni);
	glMaterialfv(GL_FRONT, GL_DIFFUSE, difuzni);
	glMaterialfv(GL_FRONT, GL_SPECULAR, odlesky);
	glMaterialf(GL_FRONT, GL_SHININESS, koefOdlesku);

	vykresliSteny();

	glPushMatrix();
	glPushAttrib(GL_ALL_ATTRIB_BITS);
	glCallList(dlist);
	glPopAttrib();
	glPopMatrix();
}
