#include <windows.h>
#include <cstdio>
#include <GL/gl.h>
#include "objekty.h"

#define MODEL_BROKOVNICE					"modely/brokovnice.obj"

///funkce na vykresleni modelu
void drawModel(char* filename, GLMmodel* model)
{
	glPushAttrib(GL_ALL_ATTRIB_BITS);
	if (model == NULL) {
		model = glmReadOBJ(filename);
		glmUnitize(model);
		glmFacetNormals(model);
		glmVertexNormals(model, 90.0);
	}
	glmDraw(model, GLM_SMOOTH | GLM_MATERIAL);
	glPopAttrib();
}

///konstruktor
Puska::Puska() : kour()
{
	model = NULL;
	uhel = poziceX = poziceY = poziceZ = 0.0;
	zvetseni = 1.0;
	vykresluj = true;
	zasobnik = NABOJU;
}

///destruktor
Puska::~Puska()
{
	if (model) free(model);
}

///pokud to je mozne puska vystreli
bool Puska::vystrel()
{
	if (vykreslujeSe() && maNaboje()) {
		zasobnik--;
		kour.zapniKour();
		return true;
	}

	return false;
}

///vykresli jeden naboj
void Puska::kresliNaboj()
{
	char retezec[128];
	memset(retezec, '\0', 128 * sizeof(char));
	sprintf(retezec, "Pocet naboju: %d", zasobnik);

	for (int i = 0; i < 128; i++) {
		if (retezec[i] == '\0')
			break;
		glutStrokeCharacter(GLUT_STROKE_ROMAN, retezec[i]);
	}
}

///vykreslovaci funkce tridy puska
void Puska::vykresli()
{
	if (vykresluj) {
		glPushMatrix();
			glTranslated(poziceX, poziceY, poziceZ);
			glScalef(zvetseni * 1.0, zvetseni * 1.0, zvetseni * 1.0);
			glRotatef(uhel, 0.0, 1.0, 0.0);
			drawModel(MODEL_BROKOVNICE, model);
		glPopMatrix();

		kour.vykresli();

		glPushMatrix();
			glEnable(GL_BLEND);
			glBlendFunc(GL_ZERO, GL_CONSTANT_COLOR);
			glColor4f(1.0, 1.0, 1.0, 1.0);
			glTranslated(poziceX-4.0, poziceY+3.5, poziceZ-4.0);
			glScalef(0.002, 0.002, 0.002);
			kresliNaboj();
			glDisable(GL_BLEND);
		glPopMatrix();
	}
}

///povoli vykresleni zbrane
void Puska::povolVykresleni()
{
	vykresluj = true;
}

///zakaze vykresleni zbrane
void Puska::zakazVykresleni()
{
	vykresluj = false;
}

///vrati true pokud se zbran vykresluje
bool Puska::vykreslujeSe()
{
	return vykresluj;
}

void Puska::zmenPozici(float x, float y, float z)
{
	poziceX = x;
	poziceY = y;
	poziceZ = z;
	kour.zmenPozici(poziceX, poziceY, poziceZ-3.0);
}

void Puska::zmenOtoceni(float otoceni)
{
	uhel = otoceni;
}

void Puska::zmenZvetseni(float scale)
{
	zvetseni = scale;
}
